@@ -34,7 +34,6 @@ static int (*gpfs_set_winattrs_fn)(int fd, int flags,
 static int (*gpfs_get_winattrs_path_fn)(char *pathname,
 					struct gpfs_winattr *attrs);
 static int (*gpfs_get_winattrs_fn)(int fd, struct gpfs_winattr *attrs);
-static int (*gpfs_prealloc_fn)(int fd, gpfs_off64_t start, gpfs_off64_t bytes);
 static int (*gpfs_ftruncate_fn)(int fd, gpfs_off64_t length);
 static int (*gpfs_lib_init_fn)(int flags);
 static int (*gpfs_set_times_path_fn)(char *pathname, int flags,
@@ -45,6 +44,10 @@ static int (*gpfs_init_trace_fn)(void);
 static int (*gpfs_query_trace_fn)(void);
 static void (*gpfs_add_trace_fn)(int level, const char *msg);
 static void (*gpfs_fini_trace_fn)(void);
+static int (*gpfs_fstat_x_fn)(int fd, unsigned int *litemask,
+			      struct gpfs_iattr64 *iattr, size_t len);
+static int (*gpfs_stat_x_fn)(const char *pathname, unsigned int *litemask,
+			     struct gpfs_iattr64 *iattr, size_t len);
 
 int gpfswrap_init(void)
 {
@@ -68,7 +71,6 @@ int gpfswrap_init(void)
 	gpfs_set_winattrs_fn	      = dlsym(l, "gpfs_set_winattrs");
 	gpfs_get_winattrs_path_fn     = dlsym(l, "gpfs_get_winattrs_path");
 	gpfs_get_winattrs_fn	      = dlsym(l, "gpfs_get_winattrs");
-	gpfs_prealloc_fn	      = dlsym(l, "gpfs_prealloc");
 	gpfs_ftruncate_fn	      = dlsym(l, "gpfs_ftruncate");
 	gpfs_lib_init_fn	      = dlsym(l, "gpfs_lib_init");
 	gpfs_set_times_path_fn	      = dlsym(l, "gpfs_set_times_path");
@@ -78,6 +80,8 @@ int gpfswrap_init(void)
 	gpfs_query_trace_fn	      = dlsym(l, "gpfs_query_trace");
 	gpfs_add_trace_fn	      = dlsym(l, "gpfs_add_trace");
 	gpfs_fini_trace_fn	      = dlsym(l, "gpfs_fini_trace");
+	gpfs_fstat_x_fn	      = dlsym(l, "gpfs_fstat_x");
+	gpfs_stat_x_fn		      = dlsym(l, "gpfs_stat_x");
 
 	return 0;
 }
@@ -173,16 +177,6 @@ int gpfswrap_get_winattrs(int fd, struct gpfs_winattr *attrs)
 	return gpfs_get_winattrs_fn(fd, attrs);
 }
 
-int gpfswrap_prealloc(int fd, gpfs_off64_t start, gpfs_off64_t bytes)
-{
-	if (gpfs_prealloc_fn == NULL) {
-		errno = ENOSYS;
-		return -1;
-	}
-
-	return gpfs_prealloc_fn(fd, start, bytes);
-}
-
 int gpfswrap_ftruncate(int fd, gpfs_off64_t length)
 {
 	if (gpfs_ftruncate_fn == NULL) {
@@ -271,3 +265,25 @@ void gpfswrap_fini_trace(void)
 
 	gpfs_fini_trace_fn();
 }
+
+int gpfswrap_fstat_x(int fd, unsigned int *litemask,
+		     struct gpfs_iattr64 *iattr, size_t len)
+{
+	if (gpfs_fstat_x_fn == NULL) {
+		errno = ENOSYS;
+		return -1;
+	}
+
+	return gpfs_fstat_x_fn(fd, litemask, iattr, len);
+}
+
+int gpfswrap_stat_x(const char *pathname, unsigned int *litemask,
+		    struct gpfs_iattr64 *iattr, size_t len)
+{
+	if (gpfs_stat_x_fn == NULL) {
+		errno = ENOSYS;
+		return -1;
+	}
+
+	return gpfs_stat_x_fn(pathname, litemask, iattr, len);
+}
