@@ -22,7 +22,6 @@
 #include "smbd/smbd.h"
 #include "system/filesys.h"
 #include "librpc/gen_ndr/xattr.h"
-#include "../lib/crypto/sha256.h"
 #include "dbwrap/dbwrap.h"
 #include "dbwrap/dbwrap_open.h"
 #include "auth.h"
@@ -50,7 +49,7 @@ static bool acl_tdb_init(void)
 		return true;
 	}
 
-	dbname = state_path("file_ntacls.tdb");
+	dbname = state_path(talloc_tos(), "file_ntacls.tdb");
 
 	if (dbname == NULL) {
 		errno = ENOSYS;
@@ -90,24 +89,6 @@ static void disconnect_acl_tdb(struct vfs_handle_struct *handle)
 	}
 }
 
-/*******************************************************************
- Fetch_lock the tdb acl record for a file
-*******************************************************************/
-
-static struct db_record *acl_tdb_lock(TALLOC_CTX *mem_ctx,
-					struct db_context *db,
-					const struct file_id *id)
-{
-	uint8_t id_buf[16];
-
-	/* For backwards compatibility only store the dev/inode. */
-	push_file_id_16((char *)id_buf, id);
-	return dbwrap_fetch_locked(db,
-				   mem_ctx,
-				   make_tdb_data(id_buf,
-						 sizeof(id_buf)));
-}
-
 /*******************************************************************
  Delete the tdb acl record for a file
 *******************************************************************/
@@ -118,20 +99,12 @@ static NTSTATUS acl_tdb_delete(vfs_handle_struct *handle,
 {
 	NTSTATUS status;
 	struct file_id id = vfs_file_id_from_sbuf(handle->conn, psbuf);
-	struct db_record *rec = acl_tdb_lock(talloc_tos(), db, &id);
+	uint8_t id_buf[16];
 
-	/*
-	 * If rec == NULL there's not much we can do about it
-	 */
+	/* For backwards compatibility only store the dev/inode. */
+	push_file_id_16((char *)id_buf, &id);
 
-	if (rec == NULL) {
-		DEBUG(10,("acl_tdb_delete: rec == NULL\n"));
-		TALLOC_FREE(rec);
-		return NT_STATUS_OK;
-	}
-
-	status = dbwrap_record_delete(rec);
-	TALLOC_FREE(rec);
+	status = dbwrap_delete(db, make_tdb_data(id_buf, sizeof(id_buf)));
 	return status;
 }
 
@@ -205,9 +178,8 @@ static NTSTATUS store_acl_blob_fsp(vfs_handle_struct *handle,
 {
 	uint8_t id_buf[16];
 	struct file_id id;
-	TDB_DATA data;
+	TDB_DATA data = { .dptr = pblob->data, .dsize = pblob->length };
 	struct db_context *db = acl_db;
-	struct db_record *rec;
 	NTSTATUS status;
 
 	DEBUG(10,("store_acl_blob_fsp: storing blob length %u on file %s\n",
@@ -222,24 +194,20 @@ static NTSTATUS store_acl_blob_fsp(vfs_handle_struct *handle,
 
 	/* For backwards compatibility only store the dev/inode. */
 	push_file_id_16((char *)id_buf, &id);
-	rec = dbwrap_fetch_locked(db, talloc_tos(),
-				  make_tdb_data(id_buf,
-						sizeof(id_buf)));
-	if (rec == NULL) {
-		DEBUG(0, ("store_acl_blob_fsp_tdb: fetch_lock failed\n"));
-		return NT_STATUS_INTERNAL_DB_CORRUPTION;
-	}
-	data.dptr = pblob->data;
-	data.dsize = pblob->length;
-	return dbwrap_record_store(rec, data, 0);
+
+	status = dbwrap_store(
+		db, make_tdb_data(id_buf, sizeof(id_buf)), data, 0);
+	return status;
 }
 
 /*********************************************************************
- On unlink we need to delete the tdb record (if using tdb).
+ On unlinkat we need to delete the tdb record (if using tdb).
 *********************************************************************/
 
-static int unlink_acl_tdb(vfs_handle_struct *handle,
-			  const struct smb_filename *smb_fname)
+static int unlinkat_acl_tdb(vfs_handle_struct *handle,
+			struct files_struct *dirfsp,
+			const struct smb_filename *smb_fname,
+			int flags)
 {
 	struct smb_filename *smb_fname_tmp = NULL;
 	struct db_context *db = acl_db;
@@ -261,7 +229,16 @@ static int unlink_acl_tdb(vfs_handle_struct *handle,
 		goto out;
 	}
 
-	ret = unlink_acl_common(handle, smb_fname_tmp);
+	if (flags & AT_REMOVEDIR) {
+		ret = rmdir_acl_common(handle,
+				dirfsp,
+				smb_fname_tmp);
+	} else {
+		ret = unlink_acl_common(handle,
+				dirfsp,
+				smb_fname_tmp,
+				flags);
+	}
 
 	if (ret == -1) {
 		goto out;
@@ -272,32 +249,6 @@ static int unlink_acl_tdb(vfs_handle_struct *handle,
 	return ret;
 }
 
-/*********************************************************************
- On rmdir we need to delete the tdb record (if using tdb).
-*********************************************************************/
-
-static int rmdir_acl_tdb(vfs_handle_struct *handle,
-		const struct smb_filename *smb_fname)
-{
-
-	SMB_STRUCT_STAT sbuf;
-	struct db_context *db = acl_db;
-	int ret = -1;
-
-	ret = vfs_stat_smb_basename(handle->conn, smb_fname, &sbuf);
-	if (ret == -1) {
-		return -1;
-	}
-
-	ret = rmdir_acl_common(handle, smb_fname);
-	if (ret == -1) {
-		return -1;
-	}
-
-	acl_tdb_delete(handle, db, &sbuf);
-	return 0;
-}
-
 /*******************************************************************
  Handle opening the storage tdb if so configured.
 *******************************************************************/
@@ -489,8 +440,7 @@ static NTSTATUS acl_tdb_fset_nt_acl(vfs_handle_struct *handle,
 static struct vfs_fn_pointers vfs_acl_tdb_fns = {
 	.connect_fn = connect_acl_tdb,
 	.disconnect_fn = disconnect_acl_tdb,
-	.rmdir_fn = rmdir_acl_tdb,
-	.unlink_fn = unlink_acl_tdb,
+	.unlinkat_fn = unlinkat_acl_tdb,
 	.chmod_fn = chmod_acl_module_common,
 	.fchmod_fn = fchmod_acl_module_common,
 	.fget_nt_acl_fn = acl_tdb_fget_nt_acl,
