@@ -202,32 +202,7 @@ struct tevent_req *wb_child_request_send(TALLOC_CTX *mem_ctx,
 
 	state->ev = ev;
 	state->child = child;
-
-	/*
-	 * We have to make a copy of "request", because our caller
-	 * might drop us via talloc_free().
-	 *
-	 * The talloc_move() magic in wb_child_request_cleanup() keeps
-	 * all the requests, but if we are sitting deep within
-	 * writev_send() down to the client, we have given it the
-	 * pointer to "request". As our caller lost interest, it will
-	 * just free "request", while writev_send still references it.
-	 */
-
-	state->request = talloc_memdup(state, request, sizeof(*request));
-	if (tevent_req_nomem(state->request, req)) {
-		return tevent_req_post(req, ev);
-	}
-
-	if (request->extra_data.data != NULL) {
-		state->request->extra_data.data = talloc_memdup(
-			state->request,
-			request->extra_data.data,
-			request->extra_len);
-		if (tevent_req_nomem(state->request->extra_data.data, req)) {
-			return tevent_req_post(req, ev);
-		}
-	}
+	state->request = request;
 
 	subreq = tevent_queue_wait_send(state, ev, child->queue);
 	if (tevent_req_nomem(subreq, req)) {
@@ -355,7 +330,6 @@ static void wb_child_request_cleanup(struct tevent_req *req,
 
 		subreq = talloc_move(state->child->queue, &state->subreq);
 		talloc_move(subreq, &state->queue_subreq);
-		talloc_move(subreq, &state->request);
 		tevent_req_set_callback(subreq,
 					wb_child_request_orphaned,
 					state->child);
@@ -783,13 +757,16 @@ void setup_child(struct winbindd_domain *domain, struct winbindd_child *child,
 		 const char *logprefix,
 		 const char *logname)
 {
+	const struct loadparm_substitution *lp_sub =
+		loadparm_s3_global_substitution();
+
 	if (logprefix && logname) {
 		char *logbase = NULL;
 
-		if (*lp_logfile(talloc_tos())) {
+		if (*lp_logfile(talloc_tos(), lp_sub)) {
 			char *end = NULL;
 
-			if (asprintf(&logbase, "%s", lp_logfile(talloc_tos())) < 0) {
+			if (asprintf(&logbase, "%s", lp_logfile(talloc_tos(), lp_sub)) < 0) {
 				smb_panic("Internal error: asprintf failed");
 			}
 
